// Demo program for communication between Processing and Arduino/Teensy
// Author : P. Peters
// Version 0.1
// Date 20170928

// use serial communication library
import processing.serial.*;

// declare and initialize brightness
int brightness = 50;

// declare serial port
Serial myPort;

// declare image
PImage img;

// perform initializations
void setup() {
  /// canvas size
  size(400, 400);
  // show serial devices for debugging purposes
  println(Serial.list());
  // instantiate serial connection
  // array index of Serial.list()[] has to match the right 
  // serial connection...
  myPort = new Serial(this, Serial.list()[2], 115200);
  // make serial event happen at line end
  myPort.bufferUntil(10);
  // give serial conection time to settle
  delay(1000);
  // load test image
  img = loadImage("testimage.jpg");
} 

void draw() {
  // display image
  image(img, 0, 0, 400, 400);
}

// return the RGB values of the pixel under the mouse as a string
// of comma separated values. Additionall the brightness is added as a 4th parameter
String sColor() {
  // get pixel color of pixel where the mouse is at
  color c = get(mouseX, mouseY);
  // turn this color into RGBI values into a comma separated string
  return nf(red(c))+","+nf(green(c))+","+nf(blue(c))+","+nf(brightness);
}

// when the mouse is mobved inside the canvas, the pixel color is sent
void mouseMoved() {
  myPort.write(sColor()+'\n');
}

// any data received will be displayed
void serialEvent(Serial p) {
  String inString = trim(p.readString());
  println(inString);
}

// keypressed influence the brightness
void keyPressed() {
  if (key == CODED) {
    switch (keyCode) {
    case UP:
      brightness=min(brightness+5, 255);
      break;
    case DOWN:
      brightness=max(brightness-5, 0);
      break;
    }
  } else {
    switch(key) {
    case '+': 
      brightness=min(brightness+5, 255);
      break;
    case '-': 
      brightness=max(brightness-5, 0);
      break;
    case '0':
      brightness = 0;
      break;
    }
  }
  // weird use of mouseMoved, but its a function like any other, so
  // use it as you see fit. Here to transmit changes in brightness
  mouseMoved();
}